/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

//! author="Jeremy Friesner"

#ifndef STRINGMATCHER_H
#define STRINGMATCHER_H

#include <sys/types.h>

#ifdef __BEOS__
# if __POWERPC__
#  include "regex/regex.h"  // use included regex if system doesn't provide one
# else
#  include <regex.h>
# endif
#else
# include <regex.h>
#endif

////////////////////////////////////////////////////////////////////////////
//
// NOTE:  This class is based on the psStringMatcher v1.3 class 
//        developed by Lars Jørgen Aas <larsa@tihlde.hist.no> for the
//        Prodigal Software File Requester.  Used by permission.
//
////////////////////////////////////////////////////////////////////////////


struct re_pattern_buffer;
typedef struct re_pattern_buffer regex_t;

//:A utility class for doing regular expression matching.  (A thin wrapper around the C regex calls)
class StringMatcher 
{
public:
    // Default ctor
    StringMatcher();

    // A constructor that sets the simple expression
    StringMatcher(const char * matchString);
    
    ~StringMatcher();

    // This is generally the method to use.  Call it once to set your
    // pattern (e.g. "Foo*"), then call Match() on each string you
    // want to test against the pattern.
    // If you wish to use the formal regex syntax, instead of the simple syntax,
    // set (isSimpleFormat) to false.
    bool SetPattern(const char * const expression, bool isSimpleFormat=true);
    
    // Returns true iff (string) is matched by the current expression.
    bool Match(const char * const string);
    
private:
    bool _regExpValid;
    regex_t _regExp;
}; 

#endif
