/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */

#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include "support/PortableMacros.h"
#include "syslog/SysLog.h"

static int _fileLogLevel    = LOG_NONE;
static int _consoleLogLevel = LOG_INFO;
static FILE * _logFile      = NULL;

static const char * _logLevelNames[] = {
   "None",
   "Critical Errors Only",
   "Errors Only",
   "Warnings and Errors Only",
   "Informational",
   "Debug",
   "Trace"
};

// This code put a #define due to the difficulty in making into a function
// (it's hard to forward varargs, ya know?)
#define LOG_BODY                              \
{                                             \
   if ((ll <= _fileLogLevel)&&(_logFile == NULL)) \
   {                                          \
      char buf[50];                           \
      time_t n = time(NULL);                  \
      struct tm * now = localtime(&n);        \
      char tb[100];                           \
      sprintf(tb, "%02i%02i%02i%02i.log", now->tm_mon+1, now->tm_mday, now->tm_hour, now->tm_min); \
      _logFile = fopen(tb, "w");              \
      if (_logFile) LogTime(LOG_INFO, "Created log file %s\n", tb); \
   }                                          \
   if (ll <= _consoleLogLevel)                \
   {                                          \
      va_list argList;                        \
      va_start(argList, fmt);                 \
      vprintf(fmt, argList);                  \
   }                                          \
   if ((ll <= _fileLogLevel)&&(_logFile))     \
   {                                          \
      va_list argList;                        \
      va_start(argList, fmt);                 \
      vfprintf(_logFile, fmt, argList);       \
      fflush(_logFile);                       \
   }                                          \
}

const char * GetLogLevelName(int ll)
{
   return ((ll>=0)&&(ll<ARRAYITEMS(_logLevelNames))) ? _logLevelNames[ll] : "???";
}

int GetFileLogLevel()
{
   return _fileLogLevel;
}

int GetConsoleLogLevel()
{
   return _consoleLogLevel;
}

int GetMaxLogLevel()
{
   return (_consoleLogLevel > _fileLogLevel) ? _consoleLogLevel : _fileLogLevel;
}

void SetFileLogLevel(int loglevel)
{
   _fileLogLevel = loglevel;
   LogTime(LOG_INFO, "File logging level set to: %s\n", GetLogLevelName(_fileLogLevel));
}

void SetConsoleLogLevel(int loglevel)
{
   _consoleLogLevel = loglevel;
   LogTime(LOG_INFO, "Console logging level set to: %s\n", GetLogLevelName(_consoleLogLevel));
}

void LogTime(int ll, const char * fmt, ...)
{
   time_t n = time(NULL);
   struct tm * now = localtime(&n);
   char timebuf[100];
   sprintf(timebuf, "[%c %02i/%02i %02i:%02i:%02i] ", GetLogLevelName(ll)[0], now->tm_mon+1, now->tm_mday, now->tm_hour, now->tm_min, now->tm_sec);
   Log(ll, timebuf);
   LOG_BODY;
}

void Log(int ll, const char * fmt, ...)
{
   LOG_BODY;
}
