/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>

#include "hashtable/PortableHashtable.h"
#include "string/PortableString.h"

uint32 SillyHashFunc(const uint32 & val);
uint32 SillyHashFunc(const uint32 & val)
{
   return val;
}

// This program exercises the PortableHashtable class.
void main(void) 
{
   PortableHashtable<PortableString, PortableString> table(PortableStringHashFunc);
   table.Put(PortableString("Hello"), PortableString("World"));
   table.Put(PortableString("Peanut Butter"), PortableString("Jelly"));
   table.Put(PortableString("Ham"), PortableString("Eggs"));
   table.Put(PortableString("Pork"), PortableString("Beans"));
   table.Put(PortableString("Slash"), PortableString("Dot"));
   table.Put(PortableString("Data"), PortableString("Mining"));  
   table.Put(PortableString("TestDouble"), PortableString("Play"));

   printf("String table has %i entries in it...\n", table.Size());

   PortableString lookup;
   if (table.Get(PortableString("Hello"), lookup)) printf("Hello -> %s\n", lookup.Cstr());
                                              else printf("Lookup 1 failed.\n");
   if (table.Get(PortableString("Peanut Butter"), lookup)) printf("Peanut Butter -> %s\n", lookup.Cstr());
                                              else printf("Lookup 2 failed.\n");


   PortableHashtableIterator<PortableString, PortableString> st = table.GetIterator();
   PortableString nextKeyString;
   PortableString nextValueString;

   while(st.GetNextKey(nextKeyString))
   {
      bool ret = table.Get(nextKeyString, nextValueString);
      printf("%i %s: %s -> %s\n", st.HasMoreKeys(), ret ? "OK" : "ERROR", nextKeyString.Cstr(), nextValueString.Cstr());
   }

   PortableHashtable<uint32, const char *> sillyTable(SillyHashFunc);
   sillyTable.Put(15, "Fifteen");
   sillyTable.Put(100, "One Hundred");
   sillyTable.Put(150, "One Hundred and Fifty");
   sillyTable.Put(200, "Two Hundred");

   const char * temp;
   sillyTable.Get(100, temp);
   sillyTable.Get(101, temp);
   printf("100 -> %s\n",temp);

   printf("Entries in sillyTable:\n");
   PortableHashtableIterator<uint32, const char *> it = sillyTable.GetIterator();

   uint32 nextKey;
   const char * nextValue;
   while(it.GetNextKey(nextKey))
   {
      bool ret = sillyTable.Get(nextKey, nextValue);
      printf("%i %s: %u -> %s\n", it.HasMoreKeys(), ret ? "OK" : "ERROR", nextKey, nextValue);
   }
   printf("XXX %i\n", it.HasMoreKeys());

   printf("... and by value:\n");
   while(it.GetNextValue(nextValue))
   {
      printf("%i value: %s\n", it.HasMoreValues(), nextValue);
   }
   printf("YYY %i\n", it.HasMoreValues());


   printf("Testing concurrent modification detection (this should cause an assertion failure)\n");
   PortableHashtableIterator<uint32, const char *> it2 = sillyTable.GetIterator();
   sillyTable.Put(250, "Crash me baby!");
   it2.GetNextKey(nextKey); 
   printf("You shouldn't ever see this message!\n");
}
