/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

/******************************************* 
   Downloaded from: http://www.mike95.com 
   Copyright (c)1997 Michael Olivero 
   All Rights Reserved 
********************************************/ 
#include "PortableString.h"
#include <ctype.h>
#include <stdio.h> 
#include <stdlib.h> 
#include <stdarg.h>

//============
//Constructors
//============
PortableString::PortableString(const char *Value)
{
   if (Value == NULL) Value = "";
   GetBuffer(_length = strlen(Value));
   strcpy(_buffer, Value);
}

PortableString::PortableString(const PortableString &Value)
{
   GetBuffer(_length = Value._length);
   strcpy(_buffer, Value._buffer);
}

char
PortableString::CharAt(uint32 loc) const
{
   return operator[](loc);
}

int
PortableString::CompareTo(const PortableString &s2) const
{
   return strcmp(_buffer, s2._buffer);
}

const PortableString &
PortableString::Concat(const PortableString &s2)
{
   return (*this) += s2;
}

const PortableString &
PortableString::operator=(const PortableString &Rhs)
{
   if (this == &Rhs)
      return *this;

   if (Rhs._length > _length)
   {
      delete [] _buffer;
      GetBuffer(Rhs._length);
   }

   _length = Rhs._length;
   strcpy(_buffer, Rhs._buffer);

   return *this;
}

const PortableString &
PortableString::operator+=(const char aChar)
{
   if (_length == _bufferLen)
      Double();

   _buffer[_length++] = aChar;
   _buffer[_length] = '\0';

   return *this;
}

PortableString
PortableString :: Prepend(const PortableString & str) const
{
   return str + (*this);
}

PortableString 
PortableString :: Append(const PortableString & str) const
{
   return (*this) + str;
}

const PortableString &
PortableString::operator+=(const PortableString &other)
{
   _length += other._length;
   if (_length > _bufferLen)
   {
      char *temp = _buffer;
      GetBuffer(_length);
      strcpy(_buffer, temp);
      delete [] temp;
   }
   strcat(_buffer, other._buffer);

   return *this;
}
 

int
PortableString::operator==(const PortableString &Rhs) const
{
   return (_length == Rhs._length && strcmp(_buffer, Rhs._buffer) == 0);
}

int
PortableString::operator!=(const PortableString &Rhs) const
{
   return (_length != Rhs.Length() || strcmp(_buffer, Rhs.Cstr()) != 0);
}

int
PortableString::operator<(const PortableString &Rhs) const
{
   return strcmp(_buffer, Rhs._buffer) < 0;
}

int
PortableString::operator>(const PortableString &Rhs) const
{
   return strcmp(_buffer, Rhs._buffer) > 0;
}

int
PortableString::operator<=(const PortableString &Rhs) const
{
   return strcmp(_buffer, Rhs._buffer) <= 0;
}

int
PortableString::operator>=(const PortableString & Rhs) const
{
   return strcmp(_buffer, Rhs._buffer) >= 0;
}

char &
PortableString::operator[](uint32 Index)
{
   verifyIndex(Index);
   return _buffer[Index];
}

char
PortableString::operator[](uint32 Index) const
{
   verifyIndex(Index);
   return _buffer[Index];
}


int
PortableString::EndsWith(const PortableString &s2) const
{ 
   if (_length < s2._length)
      return 0;
 
   return strcmp(&_buffer[_length - s2._length], s2.Cstr()) == 0; 
}

int
PortableString::Equals(const PortableString &s2) const
{
   return (_length == s2._length && strcmp(_buffer,s2._buffer) == 0);
}


int
PortableString::EqualsIgnoreCase(const PortableString &s2) const
{ 
   if (this == &s2) 
      return 1; 
   else if (_length != s2._length)
      return 0;
 
   return strcmp(ToLowerCase().Cstr(), s2.ToLowerCase().Cstr()) == 0;
}
 
void 
PortableString::Format(const char* fmt, ...) 
{ 
   va_list iterator; 
   va_start(iterator, fmt); 
 
 
   va_end(iterator); 
} 

void
PortableString::Replace(char findChar, char replaceChar)
{ 
   char* temp = strchr(_buffer, findChar); 
 
   if (temp != NULL) 
      *temp = replaceChar;
}
   
void
PortableString::Replace(const PortableString& match, const PortableString& replace)
{
   PortableString temp = _buffer, newString;
 
   int loc; 
   while ((loc = temp.IndexOf(match)) != -1)
   {
      newString += temp.Substring(0, loc);
      newString += replace;
      temp = temp.Substring(loc + match._length);
   } 

   newString += temp;               //get rest of string;

   *this = newString;
}

int
PortableString::IndexOf(char temp) const
{ 
   return IndexOf(temp, 0);
}

int
PortableString::IndexOf(char ch, uint32 fromIndex) const
{
   if (fromIndex >= _length)
      return -1;
 
   const char* temp = strchr(&_buffer[fromIndex], ch); 
   if (temp == NULL) 
      return -1; 
 
   return temp - _buffer;
}

int
PortableString::IndexOf(const PortableString &s2) const
{
   return IndexOf(s2, 0);
}


int
PortableString::IndexOf(const PortableString &s2, uint32 fromIndex) const
{
   if (fromIndex >= _length)
      return -1;

   const char *theFind = strstr(&_buffer[fromIndex], s2.Cstr());
 
   if (theFind == NULL) 
      return -1; 
 
   return theFind - _buffer; //pointer substraction
}



int
PortableString::LastIndexOf(char theChar) const
{
   return LastIndexOf(theChar, _length - 1);
}

int
PortableString::LastIndexOf(char ch, uint32 fromIndex) const
{
   if (fromIndex >= _length)
      return -1;
 
   char tempchar = _buffer[fromIndex + 1]; 
   _buffer[fromIndex + 1] = '\0'; 
   char* temp = strrchr(_buffer, ch); 
   _buffer[fromIndex + 1] = tempchar; 
 
   if (temp == NULL) 
      return -1;
 
   return temp - _buffer;
}

int
PortableString::LastIndexOf(const PortableString &s2) const
{
   return LastIndexOf(s2, _length - s2._length);
}

int
PortableString::LastIndexOf(const PortableString &s2, uint32 fromIndex) const
{
   //=============================
   //avoid check for empty strings
   //=============================
   if (s2._length == 0 || s2._length - 1 > fromIndex ||  
      fromIndex >= _length)
   return -1;
 
   //========================
   //matching first character
   //========================
   char temp = s2[0];

   for (int i = fromIndex; i >= 0; i--) 
   {
      if (_buffer[i] == temp &&
         (*this).Substring(i, i + s2._length).Equals(s2))
         return i;
   }
   return -1;
}

int
PortableString::StartsWith(const PortableString &s2) const
{
   if (_length < s2._length)
      return 0;
 
   return StartsWith(s2, 0); 
} 
 
int 
PortableString::StartsWith(const PortableString &s2, uint32 offset) const 
{ 
   if (offset > _length - s2._length) 
      return 0; 
 
   return strncmp(&_buffer[offset], s2.Cstr(), s2._length) == 0; 
}

PortableString
PortableString::Substring(uint32 left) const
{
   return Substring(left, _length);
}

PortableString
PortableString::Substring(uint32 left, uint32 right) const
{
   if (left > right)
   {
      int temp = right;
      right = left;
      left = temp;
   }
 
   ASSERT(right <= _length, "Index out of bounds in PortableString.Substring()");

   char temp = _buffer[right]; //save the replaced character
   _buffer[right] = '\0';     //nullify the the character
 
   PortableString outPut = (_buffer + left); //Pointer arithmetic

   _buffer[right] = temp;     //restore character

   return outPut;
}

PortableString
PortableString::ToLowerCase() const
{
   PortableString temp = _buffer;

   for (uint32 i = 0; i < _length; i++)
      temp._buffer[i] = tolower(temp._buffer[i]);

   return temp;
}

PortableString
PortableString::ToUpperCase() const
{
   PortableString temp = _buffer;

   for (uint32 i = 0; i < _length; i++)
      temp._buffer[i] = toupper(temp._buffer[i]);

   return temp;
}
 
PortableString 
PortableString::Trim() const 
{ 
   PortableString temp = _buffer; 
   uint32 i,j; 
 
   for (i = 0; i < _length; i++) 
   { 
      if (!isspace(_buffer[i])) 
         break; 
   } 
 
   for (j = temp._length - 1; j > i; j--) 
   { 
      if (!isspace(_buffer[j])) 
         break; 
   } 
 
   return temp.Substring(i, j + 1); 
} 

size_t PortableString :: FlattenedSize() const
{
   // plus one for the trailing NUL (we don't rely on this NUL internally,
   // but having it makes flattened PortableStrings identical to (const char *)'s,
   // which can be handy
   return _length+1;   
}

void PortableString :: Flatten(uint8 *buffer) const
{
   strcpy((char *)buffer, _buffer);
}

status_t PortableString :: Unflatten(const uint8 *buf, size_t size)
{
   delete [] _buffer;
   GetBuffer(_length = size-1);
   memcpy(_buffer, buf, _length);
   _buffer[_length] = '\0';  // make sure the string is terminated
   return B_NO_ERROR;  
}


/*--- ElfHash --------------------------------------------------- 
 *  The published hash algorithm used in the UNIX ELF format 
 *  for object files. Accepts a pointer to a string to be hashed 
 *  and returns an unsigned long. 
 *  jaf:  stolen from: http://www.ddj.com/articles/1996/9604/9604b/9604b.htm?topic=algorithms
 *-------------------------------------------------------------*/ 
uint32 PortableStringHashFunc(const PortableString & pstr)
{ 
    return CStringHashFunc(pstr.Cstr());
}

uint32 CStringHashFunc(const char * const & n)  // if you can read that, you know too much C++
{
    uint32 h = 0, g; 
    const unsigned char * name = (const unsigned char *) n;
    while (*name) 
    { 
        h = (h << 4) + *name++; 
        if (g = h & 0xF0000000) h ^= g >> 24; 
        h &= ~g; 
    } 
    return h; 
}

bool CStringCompareFunc(const char * const & s1, const char * const & s2)
{
    return(strcmp(s2, s2) == 0);
}


#if TEST_PORTABLE_STRING_CLASS
void main(void)
{
   printf("Hello, world!\n");
   PortableString s1("Hello,");
   PortableString s2(" World!");
   PortableString s3 = s1 + s2;
   printf("%s\n", s3.Cstr());
}
#endif
